/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.introspection.details;

import java.util.HashMap;

import org.fortiss.tooling.kernel.introspection.IIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.introspection.IIntrospectionItem;
import org.fortiss.tooling.kernel.introspection.items.ConstraintCheckerServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.introspection.items.PersistencyServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.introspection.items.PrototypeServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.ui.internal.introspection.details.handler.ConstraintCheckerServiceIntrospectionDetailsUIHandler;
import org.fortiss.tooling.kernel.ui.internal.introspection.details.handler.PersistencyServiceIntrospectionDetailsUIHandler;
import org.fortiss.tooling.kernel.ui.internal.introspection.details.handler.PrototypeServiceIntrospectionDetailsUIHandler;

/**
 * Registry class to provide the details GUI for a given {@link IIntrospectionItem}.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public final class KISSDetailsUIRegistry {

	/** Returns the singleton instance of the service. */
	public static final KISSDetailsUIRegistry INSTANCE = new KISSDetailsUIRegistry();

	/** The registry. */
	private final HashMap<Class<? extends IIntrospectionDetailsItem>, DetailsUIHandlerBase> registry =
			new HashMap<Class<? extends IIntrospectionDetailsItem>, DetailsUIHandlerBase>();

	/** Constructor. */
	public KISSDetailsUIRegistry() {
		registerHandler(ConstraintCheckerServiceIntrospectionDetailsItem.class,
				new ConstraintCheckerServiceIntrospectionDetailsUIHandler());
		registerHandler(PrototypeServiceIntrospectionDetailsItem.class,
				new PrototypeServiceIntrospectionDetailsUIHandler());
		registerHandler(PersistencyServiceIntrospectionDetailsItem.class,
				new PersistencyServiceIntrospectionDetailsUIHandler());
	}

	/** Registers the given composite */
	public void registerHandler(Class<? extends IIntrospectionDetailsItem> clazz,
			DetailsUIHandlerBase detailsHandler) {
		registry.put(clazz, detailsHandler);
	}

	/** Returns the composite factory for the given class. */
	public DetailsUIHandlerBase getHandler(Class<? extends IIntrospectionDetailsItem> clazz) {
		return registry.get(clazz);
	}
}
