/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.introspection.details.handler;

import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.dialogs.FilteredTree;
import org.eclipse.ui.dialogs.PatternFilter;
import org.fortiss.tooling.kernel.ui.internal.introspection.details.DetailsUIHandlerBase;

/**
 * Base class for details UI implementations with filtered tree viewer.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public abstract class FilteredTreeIntrospectionDetailsUIHandlerBase extends DetailsUIHandlerBase {
	/** Creates the filtered tree control. */
	protected final FilteredTree createFilteredTree(Composite parent) {
		FilteredTree filteredTree =
				new FilteredTree(parent, SWT.BORDER, createPatternFilter(), true);

		TreeViewer treeViewer = filteredTree.getViewer();
		Tree tree = treeViewer.getTree();
		tree.setHeaderVisible(true);
		tree.setLinesVisible(true);

		createTreeColumns(tree);

		treeViewer.setContentProvider(createContentProvider());
		treeViewer.setLabelProvider(createLabelProvider());
		treeViewer.setSorter(createSorter());
		treeViewer.setInput(getInputObject());
		return filteredTree;
	}

	/** Create the columns of the tree. */
	protected abstract void createTreeColumns(Tree tree);

	/** Creates the content provider. */
	protected abstract ITreeContentProvider createContentProvider();

	/** Creates the label provider. */
	protected abstract ITableLabelProvider createLabelProvider();

	/** Creates the pattern filter. */
	protected abstract PatternFilter createPatternFilter();

	/** Returns the input object for the tree viewer. */
	protected abstract Object getInputObject();

	/**
	 * Creates the sorter for the filtered tree. The default is {@link ViewerSorter#ViewerSorter()}.
	 */
	protected ViewerSorter createSorter() {
		return new ViewerSorter();
	}
}
